
import socket
import struct
import ctypes
import time
libhowl = ctypes.CDLL('/usr/lib/libhowl.so')


class SW_IPV4_ADDR(ctypes.Structure):
    _fields_ = [('m_addr', ctypes.c_int)]

# Callback function pointer type
SW_RESULT = ctypes.CFUNCTYPE(ctypes.c_int, # return value
                             ctypes.c_void_p, # sw_discovery
                             ctypes.c_int,    # id
                             ctypes.c_int,    # status
                             ctypes.c_int,    # interface_index
                             ctypes.c_char_p, # name
                             ctypes.c_char_p, # type
                             ctypes.c_char_p, # domain
                             ctypes.c_void_p) # extra 

SW_DISCOVERY_RESOLVE_REPLY = ctypes.CFUNCTYPE(ctypes.c_int, # return value
                                              ctypes.c_void_p, # discovery
                                              ctypes.c_int,    # oid
                                              ctypes.c_int,    # interface
                                              ctypes.c_char_p, # name
                                              ctypes.c_char_p, # type
                                              ctypes.c_char_p, # domain
                                              SW_IPV4_ADDR,    # ip
                                              ctypes.c_short,    # port
                                              ctypes.c_char_p, # octets
                                              ctypes.c_int,    # len
                                              ctypes.c_void_p) # extra

class Discovery:
    def __init__(self):
        self.session = ctypes.c_void_p()
        res = libhowl.sw_discovery_init(ctypes.byref(self.session))
        if not (res == 0):
            raise Exception, "Failed to initialize the DNS-SD session"
        self.cb_data = []

    def Browse(self, query, timeout_msec = 1000, domain="local"):
        CB = SW_RESULT(self.browse_callback)
        oid = ctypes.c_int(0)
        res = libhowl.sw_discovery_browse(self.session, 
                                          0, 
                                          query, 
                                          domain, 
                                          CB, 
                                          None, 
                                          ctypes.byref(oid))
        if not (res == 0):
            raise Exception, "DNS-SD query failed."
        
        salt = ctypes.c_void_p()
        res = libhowl.sw_discovery_salt(self.session, ctypes.byref(salt))

        if not (res == 0):
            raise Exception, "salt failed\n"

        timeout = ctypes.c_int(timeout_msec)

        end_time = time.time() + timeout_msec * .001

        while (time.time() < end_time):
            timeout = ctypes.c_int( int((end_time - time.time()) * 1000) )
            libhowl.sw_salt_step(salt, ctypes.byref(timeout))

        #libhowl.sw_discovery_fina(self.session)
        retval = self.cb_data
        self.cb_data = []
        return retval
       
    def Publish(self, domain, name, type, port):
       res = libhowl.sw_discovery_publish(self.session, 
                                          0, 
                                          name, 
                                          type, 
                                          "local", 
                                          None, 
                                          port, 
                                          "", 0, 
                                          CB, None, 
                                          ctypes.byref(oid))
        

    def browse_callback(self, session, id, status, interface, name, type, domain, extra):
        #print "args:", args

        oid = ctypes.c_int(0)

        RES_CB = SW_DISCOVERY_RESOLVE_REPLY(self.resolve_callback)

        res = libhowl.sw_discovery_resolve(
            session,
            0,
            name,
            type,
            domain,
            RES_CB,
            None,
            ctypes.byref(oid));

        #self.cb_data.append((name, type, domain))

        return 0

    def resolve_callback(self, session, oid, interface, name, type, domain, \
                         address, port, text_record, text_record_len, extra) :

        ip = socket.inet_ntoa(struct.pack('l', address.m_addr))
        try:
            hostname = socket.gethostbyaddr(ip)[0]
        except socket.herror:
            hostname = ip

        self.cb_data.append((name, type, domain, hostname, port, text_record))

        return 0

